package xyz.lychee.lagfixer.libs.adventure.text;

import java.util.function.Function;
import java.util.function.Predicate;
import xyz.lychee.lagfixer.libs.adventure.builder.AbstractBuilder;
import xyz.lychee.lagfixer.libs.adventure.text.format.Style;
import xyz.lychee.lagfixer.libs.adventure.util.Buildable;
import xyz.lychee.lagfixer.libs.annotations.Contract;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;
import xyz.lychee.lagfixer.libs.annotations.ApiStatus.NonExtendable;
import xyz.lychee.lagfixer.libs.examination.Examinable;

@NonExtendable
public interface JoinConfiguration extends Buildable<JoinConfiguration, Builder>, Examinable {
   @NotNull
   static @NotNull Builder builder() {
      return new JoinConfigurationImpl.BuilderImpl();
   }

   static @NotNull JoinConfiguration noSeparators() {
      return JoinConfigurationImpl.NULL;
   }

   static @NotNull JoinConfiguration newlines() {
      return JoinConfigurationImpl.STANDARD_NEW_LINES;
   }

   static @NotNull JoinConfiguration spaces() {
      return JoinConfigurationImpl.STANDARD_SPACES;
   }

   static @NotNull JoinConfiguration commas(final boolean spaces) {
      return spaces ? JoinConfigurationImpl.STANDARD_COMMA_SPACE_SEPARATED : JoinConfigurationImpl.STANDARD_COMMA_SEPARATED;
   }

   static @NotNull JoinConfiguration arrayLike() {
      return JoinConfigurationImpl.STANDARD_ARRAY_LIKE;
   }

   static @NotNull JoinConfiguration separator(final @Nullable ComponentLike separator) {
      return (JoinConfiguration)(separator == null ? JoinConfigurationImpl.NULL : (JoinConfiguration)builder().separator(separator).build());
   }

   static @NotNull JoinConfiguration separators(final @Nullable ComponentLike separator, final @Nullable ComponentLike lastSeparator) {
      return (JoinConfiguration)(separator == null && lastSeparator == null ? JoinConfigurationImpl.NULL : (JoinConfiguration)builder().separator(separator).lastSeparator(lastSeparator).build());
   }

   @Nullable Component prefix();

   @Nullable Component suffix();

   @Nullable Component separator();

   @Nullable Component lastSeparator();

   @Nullable Component lastSeparatorIfSerial();

   @NotNull Function<ComponentLike, Component> convertor();

   @NotNull Predicate<ComponentLike> predicate();

   @NotNull Style parentStyle();

   public interface Builder extends AbstractBuilder<JoinConfiguration>, Buildable.Builder<JoinConfiguration> {
      @Contract("_ -> this")
      @NotNull
      @NotNull Builder prefix(final @Nullable ComponentLike prefix);

      @Contract("_ -> this")
      @NotNull
      @NotNull Builder suffix(final @Nullable ComponentLike suffix);

      @Contract("_ -> this")
      @NotNull
      @NotNull Builder separator(final @Nullable ComponentLike separator);

      @Contract("_ -> this")
      @NotNull
      @NotNull Builder lastSeparator(final @Nullable ComponentLike lastSeparator);

      @Contract("_ -> this")
      @NotNull
      @NotNull Builder lastSeparatorIfSerial(final @Nullable ComponentLike lastSeparatorIfSerial);

      @Contract("_ -> this")
      @NotNull
      @NotNull Builder convertor(final @NotNull Function<ComponentLike, Component> convertor);

      @Contract("_ -> this")
      @NotNull
      @NotNull Builder predicate(final @NotNull Predicate<ComponentLike> predicate);

      @Contract("_ -> this")
      @NotNull
      @NotNull Builder parentStyle(final @NotNull Style parentStyle);
   }
}
